from sigur_interact import functions
from sigur_interact.tests import settings_test


print('Создание подключения')
sigur_socket = functions.get_connection(settings_test.contr_ip, settings_test.contr_port)
print('Авторизация')
auth_result = functions.auth_connection(sigur_socket)
print("Результат:", auth_result)


from sigur_interact.main import SigurSDK
from sigur_interact.tests import settings_test as s
import unittest
import time


class TestSigur(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args,**kwargs)
        self.sigur_sdk = SigurSDK(s.contr_ip, s.contr_port, 3, 1, 4, 2, test_mode=False)

    def test_get_status(self):
        while True:
            response = self.sigur_sdk.get_point_status_parsed(3)
            print(response)
            time.sleep(0.5)



from sigur_interact import settings
from socket import socket
from traceback import format_exc


def get_connection(ip, port, test_mode, *args, **kwargs):
    if test_mode:
        from sigure_emulator.main import SigurEmulator
        se = SigurEmulator()
        return se
    try:
        sock = socket()
        sock.connect((ip, port))
        return sock
    except ConnectionRefusedError:
        pass


def new_event_react(data, external_event_func, *args, **kwargs):
    """ Реакция на новое событие по подписке """
    if external_event_func:         # Если задана функция внешнего модуля,
        external_event_func(data)   # вызвать его и передать ему событие


def send_sigur_command(sigur_socket, command):
    """ Отправляет команду в контроллер и возрващает ответ """
    command = get_command_formatted(command)
    send_command(sigur_socket, command)
    response = get_response(sigur_socket, command=command)
    return response


def get_command_formatted(command):
    """ Получает команду, форматирует его необходимый контроллеру вид и возвращает результат """
    masked_command = settings.controller_comands_mask.format(command)
    formatted_command = bytes(masked_command, encoding='utf-8')
    return formatted_command


def get_response(sock, command=None):
    """ Получает ответ от контроллера """
    try:
        return {'command': command, 'status': True, 'info': sock.recv(1024)}
    except AttributeError:
        return {'command': command, 'status': False, 'info': format_exc()}


def send_command(sock, command):
    """ Отправить команду на контролер """
    try:
        sock.send(command)
    except AttributeError:
        return {'status': False, 'info': format_exc()}


def get_sigur_auth_command(version, login, password):
    command = settings.sigur_auth_command.format(version, login, password)
    return command


def auth_connection(sock, login="Administrator", password="", version=1.8):
    """ Отправить команду на авторизацию на контроллере """
    if sock:
        auth_command = get_sigur_auth_command(version, login, password)
        send_result = send_sigur_command(sock, auth_command)
        return send_result

def get_gate_status(sock, gate_num):
    object_status = get_object_status(sock, gate_num)['info']
    status = parse_object_status(object_status)
    return status


def get_object_status(sock, object_num):
    command = settings.get_object_command.format(object_num)
    response = send_sigur_command(sock, command)
    return response


def get_object_status_parsed(sock, object_num):
    status = get_object_status(sock, object_num)
    return parse_object_status(status)


def parse_object_status(object_status):
    """ Парсит ответ от контроллера Sigur на комманду GETAPINFO """
    object_status_str = str(object_status)
    splitted = object_status_str.split(' ')
    for el in splitted:
        if 'ONLINE' in el:
            return el





from time import sleep


def unsubscribe_ce(sock):
    '''Отписка от событий контроллера класса CE'''
    sleep(1)
    sock.send(b'"UNSUBSCRIBE" CE\r\n')


def unsubscribe_us(sock):
    '''Отписка от обычный событий контроллера'''
    sleep(1)
    sock.send(b'"UNSUBSCRIBE"\r\n')


def subscribe_ce(sock):
    sleep(1)
    sock.send(b'"SUBSCRIBE" CE\r\n')


def subscribe_us(sock):
    sleep(1)
    sock.send(b'"SUBSCRIBE"\r\n')


def resubscribe(sock):
    print('\nПереподписка')
    unsubscribe_ce(sock)
    subscribe_ce(sock)


def unsubscribe_us2(sock):
    '''Отписка без получения ответа'''
    sleep(1)
    sock.send(b'"UNSUBSCRIBE"\r\n')


def get_point_status(data):
    point = data[4]
    status = data[3]
    return point, status


def send_open_gate_command(sock, gate_num):
    set_status_unlocked(sock, gate_num)


def send_close_gate_command(sock, gate_num):
    set_status_locked(sock, gate_num)


def set_status_unlocked(sock, point_num):
    set_status(sock, point_num, 'UNLOCKED')


def set_status_locked(sock, point_num):
    set_status(sock, point_num, 'LOCKED')


def set_status_normal(sock, point_num):
    set_status(sock, point_num, 'NORMAL')


def set_status(sock, point_num, state):
    msg = 'SETAPMODE {} {}\r\n'.format(state, point_num)
    sock.send(bytes(msg, encoding='utf-8'))


def send_auth_connection(sock):
    try:
        sock.send(b'"LOGIN" 1.8 "Administrator" ""\r\n')
        return sock
    except AttributeError:
        pass



from sigur_interact import commands
from sigur_interact import functions
from sigur_interact import settings
from threading import Thread


class SigurSDK:
    def __init__(self, ip, port, entry_gate_num, entry_ph_num, exit_gate_num, exit_ph_num, external_module=None,
                 version=1.8, login="Administrator", password="", test_mode=False, *args, **kwargs):
        self.entry_gate_num = entry_gate_num
        self.entry_ph_num = entry_ph_num
        self.exit_gate_num = exit_gate_num
        self.exit_ph_num = exit_ph_num
        self.all_elements = [self.entry_gate_num, self.entry_ph_num, self.exit_gate_num, self.exit_ph_num]
        self.sock = functions.get_connection(ip, port, test_mode)
        self.auth_status = functions.auth_connection(self.sock, login, password, version)
        self.external_sys = external_module      # Внешний модуль, функции которого могут вызываться из этого класса
        self.external_event_react_func = None    # Внешняя функция, которая будет вызываться по факту нового события
        self.listen_stream = []                  # В этом списке будут храниться все события с контроллера

    def open_entry_gate(self):
        commands.send_open_gate_command(self.sock, self.entry_gate_num)

    def close_entry_gate(self):
        commands.send_close_gate_command(self.sock, self.entry_gate_num)

    def open_exit_gate(self):
        commands.send_open_gate_command(self.sock, self.exit_gate_num)

    def close_exit_gate(self):
        commands.send_close_gate_command(self.sock, self.exit_gate_num)

    def set_point_state(self, point_num, state):
        commands.set_status(self.sock, point_num, state)

    def set_point_locked(self, point_num):
        commands.set_status_locked(self.sock, point_num)

    def set_point_unlocked(self, point_num):
        commands.set_status_unlocked(self.sock, point_num)

    def set_point_normal(self, point_num):
        commands.set_status_normal(self.sock, point_num)

    def subscribe_ce(self):
        commands.subscribe_ce(self.sock)

    def unsubscribe_ce(self):
        commands.unsubscribe_ce(self.sock)

    def subscribe_us(self):
        commands.subscribe_us(self.sock)

    def unsubscribe_us(self):
        commands.unsubscribe_us(self.sock)

    def start_listen_thread(self):
        """ Запустить поток параллельный поток сохранения всех событий в спсиок """
        Thread(target=self.start_listen_stream, args=()).start()

    def set_external_react_function(self, function):
        """ Установить функцию внешнего модуля, который будет вызываться при получении нового события скуд.
        Ему будет передаваться само событие"""
        self.external_event_react_func = function

    def start_listen_stream(self):
        """ Функция сохранения всех событий в список data """
        while True:
            data = self.sock.recv(1024)
            self.listen_stream.append(data)
            functions.new_event_react(data=data, external_event_func=self.external_event_react_func)

    def get_elements_status(self):
        all_el_status = {}
        for el in self.all_elements:
            status = functions.get_gate_status(self.sock, el)
            all_el_status[el] = status
        return all_el_status

    def get_point_status_parsed(self, point_num):
        return functions.get_object_status_parsed(self.sock, point_num)

    def get_gate_status(self, object_num):
        status = functions.get_gate_status(self.sock, object_num)
        return status

    def get_point_status(self, point_num):
        return functions.get_object_status(self.sock, point_num)

    def get_auth_status(self):
        return self.auth_status

    def get_gates_status(self):
        pass







from setuptools import setup, find_packages
from os.path import join, dirname

setup(
    name='sigur_interact',
    version='1.7',
    packages=find_packages(),
    author='PunchyArchy',
    author_email='ksmdrmvscthny@gmail.com',
    long_description=open(join(dirname(__file__), 'README.txt')).read(),
    install_requires=[
        'sigur_emulator',
    ],
)



from sigur_interact.tests.make_connection_test import sigur_socket
from sigur_interact import functions

all_points = ['1', '2', '3', '4']
for point in all_points:
    response = functions.get_gate_status(sigur_socket, point)
    print(response)



class SigurEmulator:
    """ Класс, эмулирующий подключение по socket с реальным контроллером СКУД, имеет методы:
    send - для отправки команд, recv - для получения ответа.
    Так-же имеет метод engine, который обрабатывает ответы полученные из send и создает ответы, доступные для recv """
    def __init__(self):
        self.buffer = ''

    def send(self, command, *args, **kwargs):
        print('\nSigurEmulator got command: {}'.format(command))


    def recv(self, *args, **kwargs):
        print("Sigur SKUD testing socket.")
        print("RECIEVING {}".format(args))

    def connect(self, *args, **kwargs):
        print("Sigur SKUD testing socket.")
        print("CONNECTING {}".format(kwargs))